<?php
namespace Omnifund\Payments\Model;
class Payment extends \Magento\Payment\Model\Method\Cc
{
    const METHOD_CODE                       = 'omnifund';

    protected $_code                    	= self::METHOD_CODE;

    protected $_omnifund;

    protected $_isGateway                   = true;
    protected $_canCapture                  = true;
    protected $_canCapturePartial           = false;
    protected $_canRefund                   = true;
    protected $_supportedCurrencyCodes = array('USD');

    protected $curlClient; 

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Omnifund\Payments\Model\Gateway $omnigateway,
        array $data = array()
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate,
            null,
            null,
            $data
        );

        $this->_code = 'omnifund';
        
      	$this->_omnifund = $omnigateway;

	//$this->curlClient = $curl;

				$this->_omnifund->setAccessKey( $this->getConfigData('gateway_login') );
				$this->_omnifund->setAccessKeySecret( $this->getConfigData('gateway_pin') );
				$this->_omnifund->setIpAddress( $_SERVER['REMOTE_ADDR'] );
				
				// Set Test Mode if necessary
				if ($this->getConfigData('gateway_debug') == 1)
					$this->_omnifund->setDebug( 'true' );
					
    }

    public function canUseForCurrency($currencyCode)
    {
        if (!in_array($currencyCode, $this->_supportedCurrencyCodes)) {
            return false;
        }
        return true;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
	{
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        try{
		 
					$this->_omnifund->setCompany( $billing->getCompany() );

					$this->_omnifund->setFirstName( $billing->getFirstName() );
					$this->_omnifund->setLastName( $billing->getLastName() );
					$this->_omnifund->setAddress1( $billing->getStreet(1) );
					$this->_omnifund->setCity( $billing->getCity() );
					$this->_omnifund->setState( $billing->getRegionCode() );
					$this->_omnifund->setZipCode( $billing->getPostcode() );
					$this->_omnifund->setCountry( $billing->getCountry() );
					$this->_omnifund->setPhone( $billing->getTelephone() );
					$this->_omnifund->setEmail( $order->getCustomerEmail() );
					
					/* 
						AS  Authorize Only 
						DS  Capture Only 
						ES  Authorize & Capture 
						CR  Credit/Refund 
						VO  Void 
						AV  AVS Check Only 
						OF  Offline (Force) 
						*/
					$this->_omnifund->setTransactionType( 'ES' );
					$this->_omnifund->setAmount( $amount * 100 );
					
					//$this->_omnifund->setInvoiceId( '1234564' );

					$this->_omnifund->setCcName( $billing->getName() );

//					$this->_omnifund->setCcType( $payment->getCcType() );
					$this->_omnifund->setCcNumber( $payment->getCcNumber() );
					$this->_omnifund->setCcExpiration( sprintf('%02d',$payment->getCcExpMonth()) . substr($payment->getCcExpYear(), 2) );
					$this->_omnifund->setCcVerification( $payment->getCcCid() );
					
					
					$this->_omnifund->setCustomerId( $order->getCustomerEmail() );
									
					$this->_omnifund->process();
					
					if ($this->_omnifund->getStatus() == "G" ) {
						// Success
						$payment->setTransactionId($this->_omnifund->getAuthCode())->setIsTransactionClosed(0);						
					} else {
						// Fail
            					$this->debugData(['exception' => $this->_omnifund->getTerminationDescription()]);
				            	//throw new \Magento\Framework\Validator\Exception(__('Payment capturing error.'));
            					throw new \Magento\Framework\Validator\Exception(__($this->_omnifund->getTerminationDescription()));
					}

          return $this;

        }catch (\Exception $e){
            $this->debugData(['exception' => $e->getMessage()]);
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage()));// . "  sent: " . $this->_omnifund->data_sent . "  input: " . $this->_omnifund->last_input));
        }
    }

    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $transactionId = $payment->getParentTransactionId();

        try {
					$this->_omnifund->setCompany( $billing->getCompany() );

					$this->_omnifund->setFirstName( $billing->getFirstName() );
					$this->_omnifund->setLastName( $billing->getLastName() );
					$this->_omnifund->setAddress1( $billing->getStreet(1) );
					$this->_omnifund->setCity( $billing->getCity() );
					$this->_omnifund->setState( $billing->getRegionCode() );
					$this->_omnifund->setZipCode( $billing->getPostcode() );
					$this->_omnifund->setCountry( $billing->getCountry() );
					$this->_omnifund->setPhone( $billing->getTelephone() );
					$this->_omnifund->setEmail( $order->getCustomerEmail() );
					
					$this->_omnifund->setTransactionType( 'CR' );
					$this->_omnifund->setAmount( $amount * 100 );
					
					//$this->_omnifund->setInvoiceId( '1234564' );

					$this->_omnifund->setCcName( $billing->getName() );

//					$this->_omnifund->setCcType( $payment->getCcType() );
					$this->_omnifund->setCcNumber( $payment->getCcNumber() );
					$this->_omnifund->setCcExpiration( sprintf('%02d',$payment->getCcExpMonth()) . substr($payment->getCcExpYear(), 2) );
					$this->_omnifund->setCcVerification( $payment->getCcCid() );
					
					
					$this->_omnifund->setCustomerId( $order->getCustomerEmail() );
									
					$this->_omnifund->process();
					
					if (($this->_omnifund->getStatus() == "G") || ($this->_omnifund->getStatus() == "R")) {
						// Success - YAY!
						
					} else {
						// Fail
            $this->debugData(['exception' => $this->_omnifund->getTerminationDescription()]);
            throw new \Magento\Framework\Validator\Exception(__('Payment capturing error.'));
					}
        } catch (\Exception $e) {
            $this->debugData(['exception' => $e->getMessage()]);
            throw new \Magento\Framework\Validator\Exception(__('Payment refunding error.'));
        }

        $payment
            ->setTransactionId($transactionId . '-' . \Magento\Sales\Model\Order\Payment\Transaction::TYPE_REFUND)
            ->setParentTransactionId($transactionId)
            ->setIsTransactionClosed(1)
            ->setShouldCloseParentTransaction(1);

        return $this;
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null){
        //$this->_minOrderTotal = $this->getConfigData('min_order_total');
        //if($quote && $quote->getBaseGrandTotal() < $this->_minOrderTotal) {
        //    return false;
        //}
        return $this->getConfigData('gateway_login', ($quote ? $quote->getStoreId() : null))
        && parent::isAvailable($quote);
    }
}